@@ -8,10 +8,11 @@
 #include <fstream>
 #include <iostream>
 #include <ostream>
+#include <sstream>
 #include <string>
+#include <iomanip>
 
 #define SPDLOG_ACTIVE_LEVEL SPDLOG_LEVEL_DEBUG
-#define SPDLOG_ENABLE_MESSAGE_COUNTER
 
 #include "spdlog/spdlog.h"
 #include "spdlog/async.h"
@@ -21,3 +22,4 @@
 #include "spdlog/sinks/ostream_sink.h"
 #include "spdlog/sinks/rotating_file_sink.h"
 #include "spdlog/sinks/stdout_color_sinks.h"
+#include "spdlog/details/pattern_formatter.h"
\ No newline at end of file
