@@ -12,7 +12,7 @@ TEST_CASE("debug and trace w/o format string", "[macros]]")
 {
 
     prepare_logdir();
-    std::string filename = "logs/simple_log";
+    std::string filename = "test_logs/simple_log";
 
     auto logger = spdlog::create<spdlog::sinks::basic_file_sink_mt>("logger", filename);
     logger->set_pattern("%v");
@@ -39,3 +39,21 @@ TEST_CASE("disable param evaluation", "[macros]")
 {
     SPDLOG_TRACE("Test message {}", throw std::runtime_error("Should not be evaluated"));
 }
+
+TEST_CASE("pass logger pointer", "[macros]")
+{
+    auto logger = spdlog::create<spdlog::sinks::null_sink_mt>("refmacro");
+    auto &ref = *logger;
+    SPDLOG_LOGGER_TRACE(&ref, "Test message 1");
+    SPDLOG_LOGGER_DEBUG(&ref, "Test message 2");
+}
+
+// ensure that even if right macro level is on- don't evaluate if the logger's level is not high enough
+// TEST_CASE("disable param evaluation2", "[macros]")
+//{
+//    auto logger = std::make_shared<spdlog::logger>("test-macro");
+//    logger->set_level(spdlog::level::off);
+//    int x = 0;
+//    SPDLOG_LOGGER_DEBUG(logger, "Test message {}", ++x);
+//    REQUIRE(x == 0);
+//}
