@@ -1,8 +1,10 @@
 #include "includes.h"
+#include "test_sink.h"
+
+using spdlog::memory_buf_t;
 
 // log to str and return it
 template<typename... Args>
-
 static std::string log_to_str(const std::string &msg, const Args &... args)
 {
     std::ostringstream oss;
@@ -61,11 +63,11 @@ TEST_CASE("color range test1", "[pattern_formatter]")
 {
     auto formatter = std::make_shared<spdlog::pattern_formatter>("%^%v%$", spdlog::pattern_time_type::local, "\n");
 
-    fmt::memory_buffer buf;
+    memory_buf_t buf;
     fmt::format_to(buf, "Hello");
-    fmt::memory_buffer formatted;
+    memory_buf_t formatted;
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, spdlog::string_view_t(buf.data(), buf.size()));
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, spdlog::string_view_t(buf.data(), buf.size()));
     formatter->format(msg, formatted);
     REQUIRE(msg.color_range_start == 0);
     REQUIRE(msg.color_range_end == 5);
@@ -76,8 +78,8 @@ TEST_CASE("color range test2", "[pattern_formatter]")
 {
     auto formatter = std::make_shared<spdlog::pattern_formatter>("%^%$", spdlog::pattern_time_type::local, "\n");
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "");
-    fmt::memory_buffer formatted;
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "");
+    memory_buf_t formatted;
     formatter->format(msg, formatted);
     REQUIRE(msg.color_range_start == 0);
     REQUIRE(msg.color_range_end == 0);
@@ -88,8 +90,8 @@ TEST_CASE("color range test3", "[pattern_formatter]")
 {
     auto formatter = std::make_shared<spdlog::pattern_formatter>("%^***%$");
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "ignored");
-    fmt::memory_buffer formatted;
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "ignored");
+    memory_buf_t formatted;
     formatter->format(msg, formatted);
     REQUIRE(msg.color_range_start == 0);
     REQUIRE(msg.color_range_end == 3);
@@ -99,9 +101,9 @@ TEST_CASE("color range test4", "[pattern_formatter]")
 {
     auto formatter = std::make_shared<spdlog::pattern_formatter>("XX%^YYY%$", spdlog::pattern_time_type::local, "\n");
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "ignored");
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "ignored");
 
-    fmt::memory_buffer formatted;
+    memory_buf_t formatted;
     formatter->format(msg, formatted);
     REQUIRE(msg.color_range_start == 2);
     REQUIRE(msg.color_range_end == 5);
@@ -112,8 +114,8 @@ TEST_CASE("color range test5", "[pattern_formatter]")
 {
     auto formatter = std::make_shared<spdlog::pattern_formatter>("**%^");
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "ignored");
-    fmt::memory_buffer formatted;
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "ignored");
+    memory_buf_t formatted;
     formatter->format(msg, formatted);
     REQUIRE(msg.color_range_start == 2);
     REQUIRE(msg.color_range_end == 0);
@@ -123,8 +125,8 @@ TEST_CASE("color range test6", "[pattern_formatter]")
 {
     auto formatter = std::make_shared<spdlog::pattern_formatter>("**%$");
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "ignored");
-    fmt::memory_buffer formatted;
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "ignored");
+    memory_buf_t formatted;
     formatter->format(msg, formatted);
     REQUIRE(msg.color_range_start == 0);
     REQUIRE(msg.color_range_end == 2);
@@ -137,62 +139,111 @@ TEST_CASE("color range test6", "[pattern_formatter]")
 TEST_CASE("level_left_padded", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%8l] %v", spdlog::pattern_time_type::local, "\n") == "[    info] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%8!l] %v", spdlog::pattern_time_type::local, "\n") == "[    info] Some message\n");
 }
 
 TEST_CASE("level_right_padded", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%-8l] %v", spdlog::pattern_time_type::local, "\n") == "[info    ] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%-8!l] %v", spdlog::pattern_time_type::local, "\n") == "[info    ] Some message\n");
 }
 
 TEST_CASE("level_center_padded", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%=8l] %v", spdlog::pattern_time_type::local, "\n") == "[  info  ] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%=8!l] %v", spdlog::pattern_time_type::local, "\n") == "[  info  ] Some message\n");
 }
 
 TEST_CASE("short level_left_padded", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%3L] %v", spdlog::pattern_time_type::local, "\n") == "[  I] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%3!L] %v", spdlog::pattern_time_type::local, "\n") == "[  I] Some message\n");
 }
 
 TEST_CASE("short level_right_padded", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%-3L] %v", spdlog::pattern_time_type::local, "\n") == "[I  ] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%-3!L] %v", spdlog::pattern_time_type::local, "\n") == "[I  ] Some message\n");
 }
 
 TEST_CASE("short level_center_padded", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%=3L] %v", spdlog::pattern_time_type::local, "\n") == "[ I ] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%=3!L] %v", spdlog::pattern_time_type::local, "\n") == "[ I ] Some message\n");
 }
 
 TEST_CASE("left_padded_short", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%3n] %v", spdlog::pattern_time_type::local, "\n") == "[pattern_tester] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%3!n] %v", spdlog::pattern_time_type::local, "\n") == "[pat] Some message\n");
 }
 
 TEST_CASE("right_padded_short", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%-3n] %v", spdlog::pattern_time_type::local, "\n") == "[pattern_tester] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%-3!n] %v", spdlog::pattern_time_type::local, "\n") == "[pat] Some message\n");
 }
 
 TEST_CASE("center_padded_short", "[pattern_formatter]")
 {
     REQUIRE(log_to_str("Some message", "[%=3n] %v", spdlog::pattern_time_type::local, "\n") == "[pattern_tester] Some message\n");
+    REQUIRE(log_to_str("Some message", "[%=3!n] %v", spdlog::pattern_time_type::local, "\n") == "[pat] Some message\n");
 }
 
 TEST_CASE("left_padded_huge", "[pattern_formatter]")
 {
-    REQUIRE(
-        log_to_str("Some message", "[%-300n] %v", spdlog::pattern_time_type::local, "\n") ==
-        "[pattern_tester                                                                                                                  ]"
-        " Some message\n");
+    REQUIRE(log_to_str("Some message", "[%-300n] %v", spdlog::pattern_time_type::local, "\n") ==
+            "[pattern_tester                                                  ] Some message\n");
+
+    REQUIRE(log_to_str("Some message", "[%-300!n] %v", spdlog::pattern_time_type::local, "\n") ==
+            "[pattern_tester                                                  ] Some message\n");
 }
 
 TEST_CASE("left_padded_max", "[pattern_formatter]")
 {
-    REQUIRE(
-        log_to_str("Some message", "[%-128n] %v", spdlog::pattern_time_type::local, "\n") ==
-        "[pattern_tester                                                                                                                  ]"
-        " Some message\n");
+    REQUIRE(log_to_str("Some message", "[%-64n] %v", spdlog::pattern_time_type::local, "\n") ==
+            "[pattern_tester                                                  ] Some message\n");
+
+    REQUIRE(log_to_str("Some message", "[%-64!n] %v", spdlog::pattern_time_type::local, "\n") ==
+            "[pattern_tester                                                  ] Some message\n");
+}
+
+// Test padding + truncate flag
+
+TEST_CASE("paddinng_truncate", "[pattern_formatter]")
+{
+    REQUIRE(log_to_str("123456", "%6!v", spdlog::pattern_time_type::local, "\n") == "123456\n");
+    REQUIRE(log_to_str("123456", "%5!v", spdlog::pattern_time_type::local, "\n") == "12345\n");
+    REQUIRE(log_to_str("123456", "%7!v", spdlog::pattern_time_type::local, "\n") == " 123456\n");
+
+    REQUIRE(log_to_str("123456", "%-6!v", spdlog::pattern_time_type::local, "\n") == "123456\n");
+    REQUIRE(log_to_str("123456", "%-5!v", spdlog::pattern_time_type::local, "\n") == "12345\n");
+    REQUIRE(log_to_str("123456", "%-7!v", spdlog::pattern_time_type::local, "\n") == "123456 \n");
+
+    REQUIRE(log_to_str("123456", "%=6!v", spdlog::pattern_time_type::local, "\n") == "123456\n");
+    REQUIRE(log_to_str("123456", "%=5!v", spdlog::pattern_time_type::local, "\n") == "12345\n");
+    REQUIRE(log_to_str("123456", "%=7!v", spdlog::pattern_time_type::local, "\n") == "123456 \n");
+
+    REQUIRE(log_to_str("123456", "%0!v", spdlog::pattern_time_type::local, "\n") == "\n");
+}
+
+TEST_CASE("paddinng_truncate_funcname", "[pattern_formatter]")
+{
+    spdlog::sinks::test_sink_st test_sink;
+
+    const char *pattern = "%v [%5!!]";
+    auto formatter = std::unique_ptr<spdlog::formatter>(new spdlog::pattern_formatter(pattern));
+    test_sink.set_formatter(std::move(formatter));
+
+    spdlog::details::log_msg msg1{spdlog::source_loc{"ignored", 1, "func"}, "test_logger", spdlog::level::info, "message"};
+    test_sink.log(msg1);
+
+    spdlog::details::log_msg msg2{spdlog::source_loc{"ignored", 1, "function"}, "test_logger", spdlog::level::info, "message"};
+    test_sink.log(msg2);
+
+    auto lines = test_sink.lines();
+    REQUIRE(lines[0] == "message [ func]");
+    REQUIRE(lines[1] == "message [funct]");
 }
 
 TEST_CASE("clone-default-formatter", "[pattern_formatter]")
@@ -200,10 +251,10 @@ TEST_CASE("clone-default-formatter", "[pattern_formatter]")
     auto formatter_1 = std::make_shared<spdlog::pattern_formatter>();
     auto formatter_2 = formatter_1->clone();
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "some message");
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "some message");
 
-    fmt::memory_buffer formatted_1;
-    fmt::memory_buffer formatted_2;
+    memory_buf_t formatted_1;
+    memory_buf_t formatted_2;
     formatter_1->format(msg, formatted_1);
     formatter_2->format(msg, formatted_2);
 
@@ -215,10 +266,10 @@ TEST_CASE("clone-default-formatter2", "[pattern_formatter]")
     auto formatter_1 = std::make_shared<spdlog::pattern_formatter>("%+");
     auto formatter_2 = formatter_1->clone();
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "some message");
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "some message");
 
-    fmt::memory_buffer formatted_1;
-    fmt::memory_buffer formatted_2;
+    memory_buf_t formatted_1;
+    memory_buf_t formatted_2;
     formatter_1->format(msg, formatted_1);
     formatter_2->format(msg, formatted_2);
 
@@ -230,10 +281,10 @@ TEST_CASE("clone-formatter", "[pattern_formatter]")
     auto formatter_1 = std::make_shared<spdlog::pattern_formatter>("%D %X [%] [%n] %v");
     auto formatter_2 = formatter_1->clone();
     std::string logger_name = "test";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "some message");
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "some message");
 
-    fmt::memory_buffer formatted_1;
-    fmt::memory_buffer formatted_2;
+    memory_buf_t formatted_1;
+    memory_buf_t formatted_2;
     formatter_1->format(msg, formatted_1);
     formatter_2->format(msg, formatted_2);
     REQUIRE(fmt::to_string(formatted_1) == fmt::to_string(formatted_2));
@@ -245,11 +296,65 @@ TEST_CASE("clone-formatter-2", "[pattern_formatter]")
     auto formatter_1 = std::make_shared<spdlog::pattern_formatter>("%D %X [%] [%n] %v", pattern_time_type::utc, "xxxxxx\n");
     auto formatter_2 = formatter_1->clone();
     std::string logger_name = "test2";
-    spdlog::details::log_msg msg(&logger_name, spdlog::level::info, "some message");
+    spdlog::details::log_msg msg(logger_name, spdlog::level::info, "some message");
 
-    fmt::memory_buffer formatted_1;
-    fmt::memory_buffer formatted_2;
+    memory_buf_t formatted_1;
+    memory_buf_t formatted_2;
     formatter_1->format(msg, formatted_1);
     formatter_2->format(msg, formatted_2);
     REQUIRE(fmt::to_string(formatted_1) == fmt::to_string(formatted_2));
 }
+
+//
+// Test source location formatting
+//
+
+#ifdef _WIN32
+static const char *test_path = "\\a\\b\\myfile.cpp";
+#else
+static const char *test_path = "/a/b//myfile.cpp";
+#endif
+
+TEST_CASE("short filename formatter-1", "[pattern_formatter]")
+{
+    spdlog::pattern_formatter formatter("%s", spdlog::pattern_time_type::local, "");
+    memory_buf_t formatted;
+    std::string logger_name = "logger-name";
+    spdlog::source_loc source_loc{test_path, 123, "some_func()"};
+    spdlog::details::log_msg msg(source_loc, "logger-name", spdlog::level::info, "Hello");
+    formatter.format(msg, formatted);
+    REQUIRE(fmt::to_string(formatted) == "myfile.cpp");
+}
+
+TEST_CASE("short filename formatter-2", "[pattern_formatter]")
+{
+    spdlog::pattern_formatter formatter("%s:%#", spdlog::pattern_time_type::local, "");
+    memory_buf_t formatted;
+    std::string logger_name = "logger-name";
+    spdlog::source_loc source_loc{"myfile.cpp", 123, "some_func()"};
+    spdlog::details::log_msg msg(source_loc, "logger-name", spdlog::level::info, "Hello");
+    formatter.format(msg, formatted);
+    REQUIRE(fmt::to_string(formatted) == "myfile.cpp:123");
+}
+
+TEST_CASE("short filename formatter-3", "[pattern_formatter]")
+{
+    spdlog::pattern_formatter formatter("%s %v", spdlog::pattern_time_type::local, "");
+    memory_buf_t formatted;
+    std::string logger_name = "logger-name";
+    spdlog::source_loc source_loc{"", 123, "some_func()"};
+    spdlog::details::log_msg msg(source_loc, "logger-name", spdlog::level::info, "Hello");
+    formatter.format(msg, formatted);
+    REQUIRE(fmt::to_string(formatted) == " Hello");
+}
+
+TEST_CASE("full filename formatter", "[pattern_formatter]")
+{
+    spdlog::pattern_formatter formatter("%g", spdlog::pattern_time_type::local, "");
+    memory_buf_t formatted;
+    std::string logger_name = "logger-name";
+    spdlog::source_loc source_loc{test_path, 123, "some_func()"};
+    spdlog::details::log_msg msg(source_loc, "logger-name", spdlog::level::info, "Hello");
+    formatter.format(msg, formatted);
+    REQUIRE(fmt::to_string(formatted) == test_path);
+}
